from triton_bert.triton_bert import TritonBert

if __name__ == "__main__":
    model = TritonBert(triton_host="30.171.160.44", model="sbert_onnx", vocab="/Users/yanyongwen712/.cache/torch/sentence_transformers/sentence-transformers_all-MiniLM-L6-v2")

    # model() == model.encodes(), use for batch inference
    vectors = model(["基金的收益率是多少？", "我有个朋友的股票天天涨停"])
    #vectors = model.encodes(["基金的收益率是多少？", "我有个朋友的股票天天涨停"])
    assert len(vectors) == 2

    # use for single inference
    vector = model.encode("基金的收益率是多少？")
    assert vectors[0] == vector



from typing import List, Optional

import torch
import torch.nn as nn
from transformers import AutoTokenizer, AutoModel


class Model4TritonServer(nn.Module):
    def __init__(self, pretrained_model:Optional[str]=None, gpu_mode: bool=False, model=None, tokenizer=None, model_eval=True):
        if pretrained_model is None and (model is None or tokenizer is None):
            raise Exception("pretrained_model and (model, tokenizer) must not be both None")
        super(Model4TritonServer, self).__init__()
        if model is None and pretrained_model is not None:
            self.model = AutoModel.from_pretrained(pretrained_model, torchscript=True)
        elif model is not None:
            self.model = model

        if tokenizer is None and pretrained_model is not None:
            self.tokenizer = AutoTokenizer.from_pretrained(pretrained_model)
        elif tokenizer is not None:
            self.tokenizer = tokenizer


        self.device = torch.device("cuda" if gpu_mode and torch.cuda.is_available() else "cpu")
        self.model = self.model.to(self.device)
        if model_eval:
            self.model.eval() # fix the Batch Normalization and Dropout

    def save_onnx(self, output_model:str="model.onnx", max_length:int=510, output_names:List[str]=['embedding'],
                  opset_version=12, do_constant_folding=True):
        # reference: https://developer.aliyun.com/article/1258483
        query = max_length * "字"
        encoded_input = self.tokenizer(query, padding=True, truncation=True, max_length=max_length, return_tensors='pt')
        inputs = [encoded_input['input_ids'], encoded_input['attention_mask']]

        input_names = ['input_ids', "attention_mask"]
        # {0: "batch_size", 1: "length"} or [0, 1] 都可以。前者表意性好
        dynamic_axes = {'input_ids': {0: "batch_size", 1: "length"}, 'attention_mask': {0: "batch_size", 1: "length"},
                        'embedding': {0: "batch_size"}}
        if 'token_type_ids' in encoded_input:
            inputs.append(encoded_input['token_type_ids'])
            input_names.append('token_type_ids')
            dynamic_axes['token_type_ids'] = [0, 1]

        torch.onnx.export(self.model, tuple(inputs),
                          output_model,
                          do_constant_folding=do_constant_folding, # 常量折叠将使用一些算好的常量来优化一些输入全为常量的节点
                          input_names=input_names,
                          output_names=output_names,
                          dynamic_axes=dynamic_axes,
                          opset_version=opset_version)

    def save_torchscript(self, output_model: str = "model.pt", max_length: int = 510, strict=False):
        query = max_length*"草"
        encoded_input = self.tokenizer(query, max_length=max_length, truncation=True, padding=True, return_tensors='pt')
        inputs = [encoded_input['input_ids'], encoded_input['attention_mask']]
        # roberta has no token_type_ids because roberta has no NSP
        if 'token_type_ids' in encoded_input:
            inputs.append(encoded_input['token_type_ids'])
        #第一个参数实际调用的是self.model.forward函数
        traced_model = torch.jit.trace(self.model, tuple(inputs), strict=strict)
        # ATTENTION：执行时是device是cpu还是gpu，这个和运行环境耦合
        torch.jit.save(traced_model, output_model)



from triton_bert.model_4_triton import Model4TritonServer

if __name__ == "__main__":
    pretrained_model = "/Users/yanyongwen712/.cache/torch/sentence_transformers/simcse-chinese-roberta-wwm-ext"
    model = Model4TritonServer(pretrained_model=pretrained_model)
    model.save_torchscript("model/simcse_model.pt")
    model.save_onnx("model/simcse_model.onnx")


from triton_bert.triton_bert import TritonBert
import numpy as np

class CrossEncoder(TritonBert):
    '''
    rank with text similarity
    '''
    def __init__(self, triton_host:str, model: str, vocab:str, **kwargs):
        super().__init__(triton_host=triton_host, model=model, vocab=vocab, **kwargs)

    def proprocess(self, triton_output):
        return np.squeeze(triton_output[0], axis=1).tolist()

    def __call__(self, query, text_pairs):
        #change user rank input into our input pairs
        texts = len(text_pairs)*[query]
        return self.predict(texts, text_pairs)

if __name__ == "__main__":
    model = CrossEncoder(triton_host="xx", model="xx", vocab="xx")
    model("小明借了小红500元", ['小红借了小明500元', '小明还了小红500元', '小明借了小红400元'])


from triton_bert.triton_bert import TritonBert
import torch.nn.functional as F
import torch

class ChitchatIntentDetection(TritonBert):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.label_list = ["闲聊", "问答", "扯淡"]

    def proprocess(self, triton_output):
        logits = triton_output[0]
        label_ids = logits.argmax(axis=-1)
        logits = torch.tensor(logits)
        probs = F.softmax(logits, dim=1).numpy()
        ret = []
        for i, label_id in enumerate(label_ids):
            prob = probs[i][label_id]
            if label_id == 2 and prob < 0.8:
                label_id = 0
            ret.append({"category": self.label_list[label_id], "confidence": float(prob)})
        return ret


import torch
from torch import Tensor

from triton_bert.model_4_triton import Model4TritonServer
from FlagEmbedding.BGE_M3 import BGEM3ForInference

class BGEM3ForTritonInference(BGEM3ForInference):

    def forward(self,
                input_ids: Tensor,
                attention_mask: Tensor,
                return_dense: bool = True,
                return_sparse: bool = False,
                return_colbert: bool = False,
                return_sparse_embedding: bool = False):
        '''
        修改入参和返回格式
        '''
        assert return_dense or return_sparse or return_colbert, 'Must choose one or more from `return_colbert`, `return_sparse`, `return_dense` to set `True`!'

        last_hidden_state = self.model(input_ids=input_ids, attention_mask=attention_mask, return_dict=True).last_hidden_state

        output = {}
        if return_dense:
            dense_vecs = self.dense_embedding(last_hidden_state, attention_mask)
            output['dense_vecs'] = dense_vecs
        if return_sparse:
            sparse_vecs = self.sparse_embedding(last_hidden_state, input_ids,
                                                return_embedding=return_sparse_embedding)
            output['sparse_vecs'] = sparse_vecs
        if return_colbert:
            colbert_vecs = self.colbert_embedding(last_hidden_state, attention_mask)
            output['colbert_vecs'] = colbert_vecs

        if self.normlized:
            if 'dense_vecs' in output:
                output['dense_vecs'] = torch.nn.functional.normalize(output['dense_vecs'], dim=-1)
            if 'colbert_vecs' in output:
                output['colbert_vecs'] = torch.nn.functional.normalize(output['colbert_vecs'], dim=-1)
        '''
         PyTorch execute failure: output must be of type Tensor, List[str] or Tuple containing one of these two types. It should not be a List / Dictionary of Tensors or a Scalar
        '''
        #return output
        # temp code. only support dense vectors
        return torch.nn.functional.normalize(dense_vecs, dim=-1) if self.normlized else dense_vecs

if __name__ == "__main__":
    pretrained_model = "BAAI/bge-m3"
    n = BGEM3ForTritonInference(pretrained_model)

    model = Model4TritonServer(model=n.model, tokenizer=n.tokenizer)
    model.save_torchscript("model/bgem3_model.pt")
    model.save_onnx("model/bgem3_model.onnx")



from typing import List

from triton_bert.triton_bert import TritonBert
import numpy as np

class Biencoder(TritonBert):
    '''
    this is sentence sbert whose vector will be stored in milvus
    '''
    def __init__(self, triton_host:str, model: str, vocab:str, **kwargs):
        super().__init__(triton_host=triton_host, model=model, vocab=vocab, **kwargs)
        self.normalize_vector = True

    def proprocess(self, triton_output):
        if self.normalize_vector:
            #if you use IP, you must normalize the vector which is the same as cosine
            return [(x /np.linalg.norm(x)).tolist() for x in triton_output[0]]
        #milvus accept list type vector
        return triton_output[0].tolist()

if __name__ == "__main__":
    model = Biencoder(triton_host="30.171.160.44", model="sbert_onnx",
                       vocab="/Users/yanyongwen712/.cache/torch/sentence_transformers/sentence-transformers_all-MiniLM-L6-v2")

    # batch inferences
    vectors = model(["基金的收益率是多少？", "我有个朋友的股票天天涨停"])
    # or
    # vectors = model.encodes(["基金的收益率是多少？", "我有个朋友的股票天天涨停"])
    assert len(vectors) == 2

    # single inference
    vector = model.encode("基金的收益率是多少？")
    assert vectors[0] == vector